"""Provider setup screen for the AceAI TUI."""

from typing import cast

from textual.app import ComposeResult
from textual.containers import Container, Horizontal
from textual.screen import ModalScreen
from textual.widgets import Button, Checkbox, Input, Label, Select, Static

from aceai.llm.openai import OpenAIModel

from .config import AceAITUIConfig, save_config

OPENAI_MODEL_OPTIONS: list[tuple[str, OpenAIModel]] = [
    ("GPT-5.1", "gpt-5.1"),
    ("GPT-5o", "gpt-5o"),
    ("GPT-5o mini", "gpt-5o-mini"),
    ("GPT-4o", "gpt-4o"),
    ("GPT-4o mini", "gpt-4o-mini"),
    ("o3 large", "o3-large"),
    ("o4 mini", "o4-mini"),
]


class ProviderSetupScreen(ModalScreen[AceAITUIConfig]):
    """Collect provider settings before the first live agent run."""

    DEFAULT_CSS = """
    ProviderSetupScreen {
        align: center middle;
    }

    #setup-panel {
        width: 72;
        height: auto;
        border: solid #88c0d0;
        padding: 1 2;
        background: #2e3440;
        color: #e5e9f0;
    }

    #setup-title {
        text-style: bold;
        margin-bottom: 1;
    }

    #setup-error {
        color: #bf616a;
        height: 1;
    }

    Input, Select, Checkbox {
        background: #3b4252;
        color: #eceff4;
        border: solid #88c0d0;
    }

    #setup-actions {
        height: auto;
        margin-top: 1;
    }
    """

    def __init__(self, *, default_model: OpenAIModel) -> None:
        super().__init__()
        self._default_model = default_model

    def compose(self) -> ComposeResult:
        with Container(id="setup-panel"):
            yield Label("AceAI provider setup", id="setup-title")
            yield Label("Provider")
            yield Select(
                [("OpenAI", "openai")],
                value="openai",
                allow_blank=False,
                id="provider",
            )
            yield Label("Model")
            yield Select(
                OPENAI_MODEL_OPTIONS,
                value=self._default_model,
                allow_blank=False,
                id="model",
            )
            yield Label("API key")
            yield Input(password=True, placeholder="OPENAI_API_KEY", id="api-key")
            yield Checkbox("Persist to ~/.aceai/config.yaml", id="persist")
            yield Static("", id="setup-error")
            with Horizontal(id="setup-actions"):
                yield Button("Start", variant="primary", id="start")
                yield Button("Quit", id="quit")

    def on_button_pressed(self, event: Button.Pressed) -> None:
        if event.button.id == "quit":
            self.app.exit()
            return
        if event.button.id != "start":
            return
        api_key = self.query_one("#api-key", Input).value
        if api_key == "":
            self.query_one("#setup-error", Static).update("API key is required")
            return
        provider = self.query_one("#provider", Select).value
        model = self.query_one("#model", Select).value
        if provider != "openai":
            raise ValueError("Unsupported provider")
        if model not in (
            "gpt-4o",
            "gpt-4o-mini",
            "gpt-5o",
            "gpt-5o-mini",
            "gpt-5.1",
            "o3-large",
            "o4-mini",
        ):
            raise ValueError("Unsupported model")
        config = AceAITUIConfig(
            provider="openai",
            api_key=api_key,
            model=cast(OpenAIModel, model),
        )
        persist = self.query_one("#persist", Checkbox).value
        if persist:
            save_config(config)
        self.dismiss(config)



from rich.panel import Panel
from rich.text import Text

from aceai.core.events import AgentEventBuilder
from aceai.llm.models import LLMResponse, LLMToolCall, LLMToolCallDelta
from aceai.core.models import AgentStep
from aceai.core.models import ToolExecutionResult
from aceai.agent.tui.events import adapt_agent_event, user_message_event
from aceai.agent.tui.widgets.stream import _render_events


def test_consecutive_assistant_deltas_render_as_one_block() -> None:
    builder = AgentEventBuilder(step_index=0, step_id="step-1")
    events = [
        adapt_agent_event(builder.llm_text_delta(text_delta="Hi")),
        adapt_agent_event(builder.llm_text_delta(text_delta="!")),
    ]

    renderables = _render_events(events)

    assert len(renderables) == 1
    panel = renderables[0]
    assert isinstance(panel, Panel)
    panel_renderable = panel.renderable
    assert isinstance(panel_renderable, Text)
    assert panel_renderable.plain == "Hi!"


def test_main_stream_renders_question_before_answer() -> None:
    builder = AgentEventBuilder(step_index=0, step_id="step-1")
    events = [
        user_message_event("How do I search?"),
        adapt_agent_event(builder.llm_started()),
        adapt_agent_event(builder.llm_text_delta(text_delta="Use rg.")),
        adapt_agent_event(
            builder.step_completed(step=AgentStep(llm_response=LLMResponse(text="Use rg.")))
        ),
        adapt_agent_event(
            builder.run_completed(
                step=AgentStep(llm_response=LLMResponse(text="Use rg.")),
                final_answer="Use rg.",
            )
        ),
    ]

    renderables = _render_events(events)

    assert len(renderables) == 2
    question = renderables[0]
    answer = renderables[1]
    assert isinstance(question, Panel)
    assert isinstance(answer, Panel)
    assert question.title == "you"
    assert answer.title == "assistant"
    question_renderable = question.renderable
    answer_renderable = answer.renderable
    assert isinstance(question_renderable, Text)
    assert isinstance(answer_renderable, Text)
    assert question_renderable.plain == "How do I search?"
    assert answer_renderable.plain == "Use rg."


def test_main_stream_omits_lifecycle_events() -> None:
    builder = AgentEventBuilder(step_index=0, step_id="step-1")
    step = AgentStep(llm_response=LLMResponse(text="done"))
    events = [
        adapt_agent_event(builder.llm_started()),
        adapt_agent_event(builder.step_completed(step=step)),
        adapt_agent_event(builder.run_completed(step=step, final_answer="done")),
    ]

    renderables = _render_events(events)

    assert renderables == []


def test_tool_call_deltas_render_as_one_collapsed_tool_message() -> None:
    builder = AgentEventBuilder(step_index=0, step_id="step-1")
    call = LLMToolCall(
        name="write_text_file",
        arguments='{"path":"binary_search.py","content":"print(1)\\n"}',
        call_id="call-1",
    )
    result = ToolExecutionResult(
        call=call,
        output='{"path":"binary_search.py","bytes_written":9}',
    )
    events = [
        adapt_agent_event(
            builder.llm_tool_call_delta(
                tool_call_delta=LLMToolCallDelta(
                    id="call-1",
                    arguments_delta='{"path":"binary',
                )
            )
        ),
        adapt_agent_event(
            builder.llm_tool_call_delta(
                tool_call_delta=LLMToolCallDelta(
                    id="call-1",
                    arguments_delta='_search.py","content":"print(1)\\n"}',
                )
            )
        ),
        adapt_agent_event(builder.tool_started(tool_call=call)),
        adapt_agent_event(builder.tool_completed(tool_call=call, tool_result=result)),
    ]

    renderables = _render_events(events)

    assert len(renderables) == 1
    panel = renderables[0]
    assert isinstance(panel, Panel)
    assert panel.title == "tool: write_text_file"
    panel_renderable = panel.renderable
    assert isinstance(panel_renderable, Text)
    assert panel_renderable.plain == "completed - file written"


def test_unknown_in_progress_tool_call_deltas_do_not_render() -> None:
    builder = AgentEventBuilder(step_index=0, step_id="step-1")
    events = [
        adapt_agent_event(
            builder.llm_tool_call_delta(
                tool_call_delta=LLMToolCallDelta(
                    id="call-1",
                    arguments_delta='{"q":',
                )
            )
        ),
        adapt_agent_event(
            builder.llm_tool_call_delta(
                tool_call_delta=LLMToolCallDelta(
                    id="call-1",
                    arguments_delta='"aceai"}',
                )
            )
        ),
    ]

    renderables = _render_events(events)

    assert renderables == []


def test_directory_tool_result_summarizes_entry_count_without_details() -> None:
    builder = AgentEventBuilder(step_index=0, step_id="step-1")
    call = LLMToolCall(
        name="list_directory",
        arguments='{"path":"."}',
        call_id="call-1",
    )
    result = ToolExecutionResult(
        call=call,
        output=(
            '{"path":".","entries":['
            '{"name":"aceai","path":"aceai","kind":"directory"},'
            '{"name":"tests","path":"tests","kind":"directory"}'
            "]}"
        ),
    )
    events = [
        adapt_agent_event(builder.tool_started(tool_call=call)),
        adapt_agent_event(builder.tool_completed(tool_call=call, tool_result=result)),
    ]

    renderables = _render_events(events)

    assert len(renderables) == 1
    panel = renderables[0]
    assert isinstance(panel, Panel)
    assert panel.title == "tool: list_directory"
    panel_renderable = panel.renderable
    assert isinstance(panel_renderable, Text)
    assert panel_renderable.plain == "completed - 2 entries"



from pathlib import Path

from ididi import Graph
from openai import AsyncOpenAI

from aceai.core import AgentBase, ToolExecutor
from aceai.llm.openai import OpenAI, OpenAIModel
from aceai.llm.service import LLMService

from .features import default_agent_tools


ACE_AGENT_SYSTEM_PROMPT = """
You are AceAI, a concise and capable agent app.

You can inspect files, edit files, search code, run shell commands, and use skills
when they match the user's task. Prefer concrete action over abstract advice.
When working in a repository, inspect the real files before changing behavior and
run the repository's tests after meaningful code changes.
"""

ACE_AGENT_SKILLS_DIR = Path(__file__).parent / "features" / "skills"


def build_ace_agent(*, api_key: str, model: OpenAIModel) -> AgentBase:
    provider = OpenAI(
        client=AsyncOpenAI(api_key=api_key),
        default_meta={"model": model},
    )
    llm_service = LLMService([provider], timeout_seconds=120.0)
    executor = ToolExecutor(Graph(), default_agent_tools())
    return AgentBase(
        prompt=ACE_AGENT_SYSTEM_PROMPT,
        default_model=model,
        llm_service=llm_service,
        executor=executor,
        skill_path=ACE_AGENT_SKILLS_DIR,
        max_steps=8,
    )




import pytest

from aceai.agent.session import SessionRecorder, SessionStore
from aceai.core.events import AgentEventBuilder
from aceai.agent.tui.app import AceAITUI
from aceai.agent.tui.demo import static_demo_events
from aceai.agent.tui.events import adapt_agent_event, user_message_event
from aceai.agent.tui.state import initial_state, reduce_events
from aceai.agent.tui.widgets import CommandInput, DetailWidget, StreamWidget, TimelineWidget


def test_reduce_events_tracks_run_completion() -> None:
    events = static_demo_events()

    state = reduce_events(events)

    assert state.status == "completed"
    assert state.final_answer == "Static TUI prototype is ready to inspect."
    assert state.error is None
    assert state.selected_event_id == events[-1].event_id


def test_reduce_events_tracks_step_and_tool_state() -> None:
    state = reduce_events(static_demo_events())

    assert len(state.steps) == 1
    step = state.steps[0]
    assert step.status == "completed"
    assert len(step.tools) == 1
    tool_state = step.tools[0]
    assert tool_state.name == "search_docs"
    assert tool_state.status == "completed"
    assert tool_state.arguments == '{"query":"aceai tui"}'
    assert tool_state.output == '{"matches":["spec/tui.md","docs/tui.md"]}'


def test_reduce_events_assigns_global_step_numbers() -> None:
    events = [
        adapt_agent_event(
            AgentEventBuilder(step_index=0, step_id="run-1-step-1").llm_started()
        ),
        adapt_agent_event(
            AgentEventBuilder(step_index=0, step_id="run-2-step-1").llm_started()
        ),
    ]

    state = reduce_events(events)

    assert [step.step_index for step in state.steps] == [0, 1]


def test_reduce_events_does_not_add_user_questions_to_timeline() -> None:
    state = reduce_events([user_message_event("What changed?")])

    assert state.steps == []
    assert state.events[0].kind == "user_message"


def test_reduce_events_does_not_add_session_notices_to_timeline() -> None:
    from aceai.agent.tui.events import session_notice_event

    state = reduce_events([session_notice_event("Sessions")])

    assert state.steps == []
    assert state.events[0].kind == "session_notice"


def test_initial_state_is_idle() -> None:
    state = initial_state()

    assert state.status == "idle"
    assert state.steps == []
    assert state.events == []


@pytest.mark.anyio
async def test_static_tui_loads_fixture_events() -> None:
    events = static_demo_events()
    app = AceAITUI(events)

    async with app.run_test():
        assert app._state.status == "completed"
        timeline = app.query_one(TimelineWidget)
        stream = app.query_one(StreamWidget)
        detail = app.query_one(DetailWidget)
        assert timeline.can_focus
        assert stream.can_focus
        assert detail.can_focus
        assert timeline.has_class("collapsed")
        assert detail.has_class("collapsed")


@pytest.mark.anyio
async def test_stream_scrolls_to_latest_content() -> None:
    builder = AgentEventBuilder(step_index=0, step_id="step-1")
    events = [user_message_event("Show me many lines")]
    for line_number in range(80):
        events.append(
            adapt_agent_event(
                builder.llm_text_delta(text_delta=f"line {line_number}\n")
            )
        )
    app = AceAITUI(events)

    async with app.run_test(size=(80, 20)) as pilot:
        await pilot.pause(0.1)
        stream = app.query_one(StreamWidget)

        assert stream.max_scroll_y > 0
        assert stream.scroll_y == stream.max_scroll_y


@pytest.mark.anyio
async def test_escape_exits_input_mode_and_returns_focus_to_stream() -> None:
    app = AceAITUI([])

    async with app.run_test() as pilot:
        command_input = app.query_one(CommandInput)
        stream = app.query_one(StreamWidget)
        command_input.focus()

        assert command_input.has_focus

        await pilot.press("escape")

        assert not command_input.has_focus
        assert stream.has_focus


@pytest.mark.anyio
async def test_tui_header_uses_session_id(tmp_path) -> None:
    store = SessionStore(tmp_path)
    metadata = store.create_session()
    app = AceAITUI(
        [],
        session_recorder=SessionRecorder(store, metadata.session_id),
        session_id=metadata.session_id,
    )

    async with app.run_test():
        assert app.title == f"AceAI {metadata.session_id}"


@pytest.mark.anyio
async def test_tui_can_show_and_switch_sessions(tmp_path) -> None:
    store = SessionStore(tmp_path)
    first = store.create_session()
    second = store.create_session()
    SessionRecorder(store, first.session_id).record(user_message_event("first question"))
    SessionRecorder(store, second.session_id).record(user_message_event("second question"))
    app = AceAITUI(
        store.load_tui_events(first.session_id),
        session_recorder=SessionRecorder(store, first.session_id),
        session_id=first.session_id,
    )

    async with app.run_test():
        app.show_sessions()

        assert app._state.events[-1].kind == "session_notice"
        assert first.session_id in app._state.events[-1].content
        assert second.session_id in app._state.events[-1].content

        app.switch_session(second.session_id)

        assert app.title == f"AceAI {second.session_id}"
        assert app._state.events[0].content == "second question"



from datetime import datetime

from aceai.agent.session import SessionRecorder, SessionStore, messages_to_llm_history
from aceai.agent.tui.events import adapt_agent_event, user_message_event
from aceai.core.events import AgentEventBuilder
from aceai.core.models import AgentStep, ToolExecutionResult
from aceai.llm.models import LLMMessage, LLMResponse, LLMToolCall, LLMToolCallDelta


def test_session_store_creates_sqlite_index_and_message_file(tmp_path) -> None:
    store = SessionStore(tmp_path)

    metadata = store.create_session()

    assert metadata.session_id != ""
    assert type(metadata.created_at) is datetime
    assert type(metadata.updated_at) is datetime
    assert (tmp_path / "sessions.sqlite3").exists()
    assert (tmp_path / "files" / f"{metadata.session_id}.jsonl").exists()
    assert store.get_session(metadata.session_id).session_id == metadata.session_id


def test_session_store_lists_sessions_by_recent_update(tmp_path) -> None:
    store = SessionStore(tmp_path)
    first = store.create_session()
    second = store.create_session()

    store.update_session_title(first.session_id, "first")

    sessions = store.list_sessions()

    assert [session.session_id for session in sessions] == [
        first.session_id,
        second.session_id,
    ]


def test_session_store_finalize_uses_first_question_and_second_level_date(
    tmp_path, monkeypatch
) -> None:
    store = SessionStore(tmp_path)
    metadata = store.create_session()
    recorder = SessionRecorder(store, metadata.session_id)
    recorder.record(user_message_event("What files are here?"))

    monkeypatch.setattr("aceai.agent.session._local_second", lambda: "2026-05-04 12:13:14")

    title = store.finalize_session_title(metadata.session_id)

    assert title == "What files are here? - 2026-05-04 12:13:14"
    assert store.get_session(metadata.session_id).title == title


def test_session_recorder_merges_streaming_assistant_deltas(tmp_path) -> None:
    store = SessionStore(tmp_path)
    metadata = store.create_session()
    recorder = SessionRecorder(store, metadata.session_id)

    recorder.record(user_message_event("hello"))
    recorder.record(
        adapt_agent_event(
            AgentEventBuilder(step_index=0, step_id="step-1").llm_text_delta(
                text_delta="hel"
            )
        )
    )
    recorder.record(
        adapt_agent_event(
            AgentEventBuilder(step_index=0, step_id="step-1").llm_text_delta(
                text_delta="lo"
            )
        )
    )
    recorder.record(
        adapt_agent_event(
            AgentEventBuilder(step_index=0, step_id="step-1").llm_completed(
                step=AgentStep(llm_response=LLMResponse(text="hello"))
            )
        )
    )

    messages = store.load_messages(metadata.session_id)

    assert [message.kind for message in messages] == ["user", "assistant"]
    assert messages[1].content == "hello"


def test_session_recorder_saves_non_streaming_llm_completion(tmp_path) -> None:
    store = SessionStore(tmp_path)
    metadata = store.create_session()
    recorder = SessionRecorder(store, metadata.session_id)

    recorder.record(
        adapt_agent_event(
            AgentEventBuilder(step_index=0, step_id="step-1").llm_completed(
                step=AgentStep(llm_response=LLMResponse(text="answer"))
            )
        )
    )

    messages = store.load_messages(metadata.session_id)

    assert len(messages) == 1
    assert messages[0].kind == "assistant"
    assert messages[0].content == "answer"


def test_session_recorder_merges_tool_deltas_into_one_message(tmp_path) -> None:
    store = SessionStore(tmp_path)
    metadata = store.create_session()
    recorder = SessionRecorder(store, metadata.session_id)
    builder = AgentEventBuilder(step_index=0, step_id="step-1")
    call = LLMToolCall(
        name="list_directory",
        arguments='{"path":"."}',
        call_id="call-1",
    )
    result = ToolExecutionResult(
        call=call,
        output=(
            '{"path":".","entries":['
            '{"name":"aceai","path":"aceai","kind":"directory"},'
            '{"name":"tests","path":"tests","kind":"directory"}'
            "]}"
        ),
    )

    recorder.record(
        adapt_agent_event(
            builder.llm_tool_call_delta(
                tool_call_delta=LLMToolCallDelta(
                    id="call-1",
                    arguments_delta='{"path"',
                )
            )
        )
    )
    recorder.record(
        adapt_agent_event(
            builder.llm_tool_call_delta(
                tool_call_delta=LLMToolCallDelta(
                    id="call-1",
                    arguments_delta=':"."}',
                )
            )
        )
    )
    recorder.record(adapt_agent_event(builder.tool_started(tool_call=call)))
    recorder.record(
        adapt_agent_event(builder.tool_completed(tool_call=call, tool_result=result))
    )

    messages = store.load_messages(metadata.session_id)

    assert len(messages) == 1
    assert messages[0].kind == "tool"
    assert messages[0].tool_name == "list_directory"
    assert messages[0].tool_arguments == '{"path":"."}'
    assert messages[0].content == "completed - 2 entries"


def test_session_store_restores_compact_messages_as_tui_events(tmp_path) -> None:
    store = SessionStore(tmp_path)
    metadata = store.create_session()
    recorder = SessionRecorder(store, metadata.session_id)
    recorder.record(user_message_event("hello"))
    recorder.record(
        adapt_agent_event(
            AgentEventBuilder(step_index=0, step_id="step-1").llm_text_delta(
                text_delta="answer"
            )
        )
    )
    recorder.flush_assistant()

    events = store.load_tui_events(metadata.session_id)

    assert [event.kind for event in events] == ["user_message", "assistant_delta"]
    assert events[0].content == "hello"
    assert events[1].content == "answer"


def test_session_messages_restore_user_assistant_llm_history_only(tmp_path) -> None:
    store = SessionStore(tmp_path)
    metadata = store.create_session()
    recorder = SessionRecorder(store, metadata.session_id)
    recorder.record(user_message_event("hello"))
    recorder.record(
        adapt_agent_event(
            AgentEventBuilder(step_index=0, step_id="step-1").llm_text_delta(
                text_delta="answer"
            )
        )
    )
    recorder.flush_assistant()

    history = messages_to_llm_history(store.load_messages(metadata.session_id))

    assert history == [
        LLMMessage.build(role="user", content="hello"),
        LLMMessage.build(role="assistant", content="answer"),
    ]


def test_session_store_exports_readable_text(tmp_path) -> None:
    store = SessionStore(tmp_path)
    metadata = store.create_session()
    recorder = SessionRecorder(store, metadata.session_id)
    recorder.record(user_message_event("hello"))
    recorder.record(
        adapt_agent_event(
            AgentEventBuilder(step_index=0, step_id="step-1").llm_text_delta(
                text_delta="answer"
            )
        )
    )
    recorder.flush_assistant()

    text = store.export_text(metadata.session_id)

    assert text.startswith(f"# AceAI session {metadata.session_id}\n")
    assert "## user\nhello\n" in text
    assert "## assistant\nanswer\n" in text



"""Main event stream pane for the read-only TUI prototype."""

from msgspec import Struct
from rich.console import RenderableType
from rich.panel import Panel
from rich.text import Text
from textual.widgets import RichLog

from aceai.agent.tui.events import TUIEvent, TUIEventKind
from aceai.agent.tui.state import TUIRunState
from aceai.agent.tui.theme import EVENT_LABELS, EVENT_STYLES


class StreamWidget(RichLog):
    """Render the readable event transcript."""

    DEFAULT_CSS = """
    StreamWidget {
        border: solid #81a1c1;
        background: #2e3440;
        color: #e5e9f0;
        padding: 0 1;
        width: 1fr;
        height: 1fr;
        overflow-y: auto;
    }
    """

    def __init__(
        self,
        state: TUIRunState | None = None,
        *,
        id: str | None = None,
    ) -> None:
        super().__init__(id=id, wrap=True, auto_scroll=True)
        self._state = state or TUIRunState()

    def set_state(self, state: TUIRunState) -> None:
        self._state = state
        self.clear()
        if not self._state.events:
            self.write(Text("No events yet", style="#d8dee9"))
        else:
            for renderable in _render_events(self._state.events):
                self.write(renderable)
        self.call_after_refresh(self.scroll_end, animate=False)


class _ToolBlockState(Struct, kw_only=True):
    call_id: str
    name: str | None = None
    arguments: str = ""
    output: str = ""
    status: str = "running"


def _render_events(events: list[TUIEvent]) -> list[RenderableType]:
    renderables: list[RenderableType] = []
    assistant_buffer = ""
    assistant_buffer_step_id = ""
    thinking_buffer = ""
    assistant_step_ids: set[str] = set()
    tool_blocks: dict[str, _ToolBlockState] = {}
    rendered_tool_call_ids: set[str] = set()

    for event in events:
        if event.kind == "assistant_delta":
            assistant_buffer_step_id = event.step_id
            assistant_buffer += event.content
            continue
        if event.kind == "thinking_delta":
            thinking_buffer += event.content
            continue

        if event.tool_call_id is not None and event.kind in (
            "tool_call_delta",
            "tool_started",
            "tool_output",
            "tool_completed",
            "tool_failed",
        ):
            assistant_buffer, assistant_buffer_step_id = _flush_assistant_buffer(
                renderables,
                assistant_buffer,
                assistant_buffer_step_id,
                assistant_step_ids,
            )
            thinking_buffer = _flush_thinking_buffer(renderables, thinking_buffer)
            _update_tool_block(tool_blocks, event)
            if event.kind in ("tool_completed", "tool_failed"):
                renderables.append(_render_tool_block(tool_blocks[event.tool_call_id]))
                rendered_tool_call_ids.add(event.tool_call_id)
            continue

        assistant_buffer, assistant_buffer_step_id = _flush_assistant_buffer(
            renderables,
            assistant_buffer,
            assistant_buffer_step_id,
            assistant_step_ids,
        )
        thinking_buffer = _flush_thinking_buffer(renderables, thinking_buffer)

        if event.kind == "llm_completed" and event.step_id in assistant_step_ids:
            continue

        rendered = _render_event(event)
        if rendered is not None:
            if event.kind == "llm_completed":
                assistant_step_ids.add(event.step_id)
            renderables.append(rendered)

    assistant_buffer, assistant_buffer_step_id = _flush_assistant_buffer(
        renderables,
        assistant_buffer,
        assistant_buffer_step_id,
        assistant_step_ids,
    )
    thinking_buffer = _flush_thinking_buffer(renderables, thinking_buffer)
    for call_id, tool_block in tool_blocks.items():
        if call_id in rendered_tool_call_ids:
            continue
        if tool_block.name is None:
            continue
        renderables.append(_render_tool_block(tool_block))
    return renderables


def _flush_assistant_buffer(
    renderables: list[RenderableType],
    assistant_buffer: str,
    assistant_buffer_step_id: str,
    assistant_step_ids: set[str],
) -> tuple[str, str]:
    if assistant_buffer == "":
        return "", assistant_buffer_step_id
    renderables.append(
        _render_text_block(
            "assistant",
            assistant_buffer,
            event_kind="assistant_delta",
        )
    )
    assistant_step_ids.add(assistant_buffer_step_id)
    return "", ""


def _flush_thinking_buffer(
    renderables: list[RenderableType],
    thinking_buffer: str,
) -> str:
    if thinking_buffer:
        renderables.append(
            _render_text_block(
                "thinking",
                thinking_buffer,
                event_kind="thinking_delta",
            )
        )
    return ""


def _update_tool_block(
    tool_blocks: dict[str, _ToolBlockState],
    event: TUIEvent,
) -> None:
    if event.tool_call_id is None:
        raise ValueError("tool event must include tool_call_id")
    tool_block = tool_blocks.get(event.tool_call_id)
    if tool_block is None:
        tool_block = _ToolBlockState(call_id=event.tool_call_id)
        tool_blocks[event.tool_call_id] = tool_block
    if event.tool_name is not None:
        tool_block.name = event.tool_name
    if event.tool_call is not None and event.tool_call.arguments != "":
        tool_block.arguments = event.tool_call.arguments
    elif event.kind == "tool_call_delta":
        tool_block.arguments += event.content
    if event.kind == "tool_output":
        tool_block.output += event.content
    elif event.kind in ("tool_completed", "tool_failed"):
        tool_block.output = event.content
    if event.kind == "tool_failed":
        tool_block.status = "failed"
    elif event.kind == "tool_completed":
        tool_block.status = "completed"


def _render_tool_block(tool_block: _ToolBlockState) -> Panel:
    event_kind: TUIEventKind = (
        "tool_failed"
        if tool_block.status == "failed"
        else "tool_completed"
        if tool_block.status == "completed"
        else "tool_started"
    )
    style = EVENT_STYLES[event_kind]
    if tool_block.name is None:
        raise ValueError("tool block must include a tool name before rendering")
    title = f"tool: {tool_block.name}"
    return Panel(
        Text(_tool_summary(tool_block), style=style),
        title=title,
        border_style=style,
    )


def _tool_summary(tool_block: _ToolBlockState) -> str:
    if tool_block.status == "failed":
        return "failed"
    if tool_block.status == "completed":
        summary = _tool_output_summary(tool_block.output)
        if summary != "":
            return f"completed - {summary}"
        return "completed"
    return "running"


def _tool_output_summary(output: str) -> str:
    if output == "":
        return ""
    if '"entries":[' in output:
        entry_count = output.count('"name"')
        return f"{entry_count} entries"
    if '"bytes_written":' in output:
        return "file written"
    if '"exit_code":0' in output:
        return "command exited 0"
    if '"exit_code":' in output:
        return "command finished"
    if '"matches":' in output:
        return "search finished"
    return "result ready"


def _render_text_block(
    label: str,
    content: str,
    *,
    event_kind: TUIEventKind,
) -> Panel:
    style = EVENT_STYLES[event_kind]
    return Panel(
        Text(content, style=style),
        title=label,
        border_style=style,
    )


def _render_event(event: TUIEvent) -> RenderableType | None:
    label = EVENT_LABELS[event.kind]
    style = EVENT_STYLES[event.kind]
    if event.kind in ("user_message", "session_notice"):
        return _render_text_block(label, event.content, event_kind=event.kind)
    if event.kind == "tool_call_delta":
        return None
    if event.kind == "assistant_delta":
        return _render_text_block(label, event.content, event_kind=event.kind)
    if event.kind in ("thinking_delta", "reasoning_summary"):
        return Panel(
            Text(event.content, style=style),
            title=label,
            border_style=style,
        )
    if event.kind in ("tool_started", "tool_completed", "tool_failed", "tool_output"):
        if event.kind == "tool_started":
            return None
        title = _tool_title(label, event)
        return Panel(
            Text(event.content or event.tool_call_id or "", style=style),
            title=title,
            border_style=style,
        )
    if event.kind == "media":
        return Panel(
            Text("media segment", style=style),
            title=label,
            border_style=style,
        )
    if event.kind == "llm_completed":
        if event.content == "":
            return None
        return _render_text_block("assistant", event.content, event_kind="assistant_delta")
    if event.kind in ("step_started", "step_completed", "run_completed"):
        return None
    return Text(f"{label}: {event.content}", style=style)


def _tool_title(label: str, event: TUIEvent) -> str:
    if event.tool_name is not None:
        return f"{label}: {event.tool_name}"
    if event.tool_call_id is not None:
        return f"{label}: {event.tool_call_id}"
    return label



from aceai.agent.tui import cli


class StubAgent:
    pass


class StubMetadata:
    session_id = "session-1"


def test_cli_question_runs_single_question_tui(monkeypatch) -> None:
    calls: list[tuple[str, object]] = []
    agent = StubAgent()

    def build_default_agent(*, api_key, model):
        calls.append(("build", (api_key, model)))
        return agent

    def run_agent_tui(received_agent, question, **kwargs):
        calls.append(("single", (received_agent, question, kwargs)))

    monkeypatch.setenv("OPENAI_API_KEY", "key")
    monkeypatch.setattr(cli, "build_default_agent", build_default_agent)
    monkeypatch.setattr(
        cli,
        "create_session_context",
        lambda *, resume_session_id: (object(), StubMetadata(), [], []),
    )
    monkeypatch.setattr(cli, "SessionRecorder", lambda store, session_id: "recorder")
    monkeypatch.setattr(cli, "run_agent_tui", run_agent_tui)

    cli.main(["hello", "world"])

    assert calls == [
        ("build", ("key", "gpt-5.1")),
        (
            "single",
            (
                agent,
                "hello world",
                {
                    "initial_events": [],
                    "initial_history": [],
                    "session_recorder": "recorder",
                    "session_id": "session-1",
                },
            ),
        ),
    ]


def test_cli_without_question_runs_interactive_tui(monkeypatch) -> None:
    calls: list[tuple[str, object]] = []
    agent = StubAgent()

    def build_default_agent(*, api_key, model):
        calls.append(("build", (api_key, model)))
        return agent

    def run_interactive_tui(received_agent, **kwargs):
        calls.append(("interactive", (received_agent, kwargs)))

    monkeypatch.setenv("OPENAI_API_KEY", "key")
    monkeypatch.setenv("ACEAI_MODEL", "gpt-4o-mini")
    monkeypatch.setattr(cli, "build_default_agent", build_default_agent)
    monkeypatch.setattr(
        cli,
        "create_session_context",
        lambda *, resume_session_id: (object(), StubMetadata(), ["event"], ["history"]),
    )
    monkeypatch.setattr(cli, "SessionRecorder", lambda store, session_id: "recorder")
    monkeypatch.setattr(cli, "run_interactive_tui", run_interactive_tui)

    cli.main([])

    assert calls == [
        ("build", ("key", "gpt-4o-mini")),
        (
            "interactive",
            (
                agent,
                {
                    "initial_events": ["event"],
                    "initial_history": ["history"],
                    "session_recorder": "recorder",
                    "session_id": "session-1",
                },
            ),
        ),
    ]


def test_cli_without_config_opens_provider_setup_tui(monkeypatch) -> None:
    calls: list[tuple[str, tuple[object, object | None, str, str, dict[str, object]]]] = []

    def run_configured_tui(
        agent_factory,
        *,
        initial_config,
        initial_question,
        default_model,
        **kwargs,
    ):
        calls.append(
            (
                "configured",
                (agent_factory, initial_config, initial_question, default_model, kwargs),
            )
        )

    monkeypatch.delenv("OPENAI_API_KEY", raising=False)
    monkeypatch.delenv("ACEAI_MODEL", raising=False)
    monkeypatch.setattr(cli, "load_config", lambda: None)
    monkeypatch.setattr(
        cli,
        "create_session_context",
        lambda *, resume_session_id: (object(), StubMetadata(), ["event"], ["history"]),
    )
    monkeypatch.setattr(cli, "SessionRecorder", lambda store, session_id: "recorder")
    monkeypatch.setattr(cli, "run_configured_tui", run_configured_tui)

    cli.main(["hello"])

    assert calls[0][0] == "configured"
    payload = calls[0][1]
    assert payload[1] is None
    assert payload[2] == "hello"
    assert payload[3] == "gpt-5.1"
    assert payload[4] == {
        "initial_events": ["event"],
        "initial_history": ["history"],
        "session_recorder": "recorder",
        "session_id": "session-1",
    }


def test_build_default_agent_uses_main_ace_agent(monkeypatch) -> None:
    calls: list[tuple[str, str]] = []
    agent = StubAgent()

    def build_ace_agent(*, api_key, model):
        calls.append((api_key, model))
        return agent

    monkeypatch.setattr(cli, "build_ace_agent", build_ace_agent)

    result = cli.build_default_agent(api_key="key", model="gpt-5.1")

    assert result is agent
    assert calls == [("key", "gpt-5.1")]


def test_cli_resume_loads_existing_session(monkeypatch) -> None:
    calls: list[tuple[str, object]] = []
    agent = StubAgent()

    def build_default_agent(*, api_key, model):
        calls.append(("build", (api_key, model)))
        return agent

    def create_session_context(*, resume_session_id):
        calls.append(("session", resume_session_id))
        return object(), StubMetadata(), ["restored"], ["history"]

    def run_interactive_tui(received_agent, **kwargs):
        calls.append(("interactive", (received_agent, kwargs)))

    monkeypatch.setenv("OPENAI_API_KEY", "key")
    monkeypatch.setattr(cli, "build_default_agent", build_default_agent)
    monkeypatch.setattr(cli, "create_session_context", create_session_context)
    monkeypatch.setattr(cli, "SessionRecorder", lambda store, session_id: "recorder")
    monkeypatch.setattr(cli, "run_interactive_tui", run_interactive_tui)

    cli.main(["resume", "session-1"])

    assert calls == [
        ("session", "session-1"),
        ("build", ("key", "gpt-5.1")),
        (
            "interactive",
            (
                agent,
                {
                    "initial_events": ["restored"],
                    "initial_history": ["history"],
                    "session_recorder": "recorder",
                    "session_id": "session-1",
                },
            ),
        ),
    ]


def test_cli_export_prints_session_text(monkeypatch, capsys) -> None:
    class StubStore:
        def export_text(self, session_id):
            assert session_id == "session-1"
            return "# AceAI session session-1\n\n## user\nhello\n"

    monkeypatch.setattr(cli, "SessionStore", StubStore)

    cli.main(["export", "session-1"])

    captured = capsys.readouterr()
    assert captured.out == "# AceAI session session-1\n\n## user\nhello\n"



"""Read-only Textual application for AceAI event streams."""

from textual.app import App, ComposeResult
from textual.containers import Horizontal
from textual.widgets import Footer, Header

from aceai.core.events import AgentEvent

from aceai.agent.session import SessionRecorder

from .events import TUIEvent
from .events import adapt_agent_event
from .events import session_notice_event
from .state import TUIRunState, apply_tui_event, initial_state, reduce_events
from .widgets import CommandInput, DetailWidget, StreamWidget, TimelineWidget


class AceAITUI(App[None]):
    """Read-only TUI prototype backed by normalized TUI events."""

    CSS = """
    Screen {
        layout: vertical;
        background: #2e3440;
        color: #e5e9f0;
    }

    Header {
        background: #3b4252;
        color: #eceff4;
        text-style: bold;
    }

    #main {
        height: 1fr;
    }

    #stream {
        min-width: 40;
    }

    TimelineWidget.collapsed {
        display: none;
    }

    DetailWidget.collapsed {
        display: none;
    }

    Footer {
        background: #3b4252;
        color: #eceff4;
    }
    """

    BINDINGS = [
        ("q", "quit", "Quit"),
        ("ctrl+c", "quit", "Quit"),
        ("e", "toggle_events", "Events"),
        ("d", "toggle_detail", "Raw Log"),
    ]

    def __init__(
        self,
        events: list[TUIEvent] | None = None,
        *,
        session_recorder: SessionRecorder | None = None,
        session_id: str | None = None,
    ) -> None:
        super().__init__()
        self._events = list(events or [])
        self._state: TUIRunState = initial_state()
        self._session_recorder = session_recorder
        self._session_id = session_id
        self.title = "AceAI" if session_id is None else f"AceAI {session_id}"

    def compose(self) -> ComposeResult:
        yield Header(show_clock=True)
        with Horizontal(id="main"):
            yield TimelineWidget(id="timeline", classes="collapsed")
            yield StreamWidget(id="stream")
            yield DetailWidget(id="detail", classes="collapsed")
        yield CommandInput(id="input")
        yield Footer()

    def on_mount(self) -> None:
        self.load_events(self._events)

    def load_events(self, events: list[TUIEvent]) -> None:
        self._state = reduce_events(events)
        self._refresh_widgets()

    def show_sessions(self) -> None:
        if self._session_recorder is None:
            self.append_event(session_notice_event("No session store is configured."))
            return
        sessions = self._session_recorder.store.list_sessions()
        if not sessions:
            self.append_event(session_notice_event("No sessions found."))
            return
        lines = ["Sessions:"]
        for session in sessions:
            marker = "*" if session.session_id == self._session_id else "-"
            lines.append(
                f"{marker} {session.session_id}  {session.title}  {session.updated_at}"
            )
        lines.append("")
        lines.append("Use /resume <session_id> to switch sessions.")
        self.append_event(session_notice_event("\n".join(lines)))

    def switch_session(self, session_id: str) -> None:
        if self._session_recorder is None:
            self.append_event(session_notice_event("No session store is configured."))
            return
        self._session_recorder.finalize()
        store = self._session_recorder.store
        metadata = store.get_session(session_id)
        self._session_recorder = SessionRecorder(store, metadata.session_id)
        self._session_id = metadata.session_id
        self.title = f"AceAI {metadata.session_id}"
        self.load_events(store.load_tui_events(metadata.session_id))
        self.append_event(session_notice_event(f"Resumed session {metadata.session_id}"))

    def append_event(self, event: TUIEvent) -> None:
        self._state = apply_tui_event(self._state, event)
        if self._session_recorder is not None:
            self._session_recorder.record(event)
        self._refresh_widgets()

    def append_agent_event(self, event: AgentEvent) -> None:
        self.append_event(adapt_agent_event(event))

    def action_toggle_detail(self) -> None:
        detail = self.query_one(DetailWidget)
        if detail.has_class("collapsed"):
            detail.remove_class("collapsed")
            detail.focus()
        else:
            detail.add_class("collapsed")
            self.query_one(StreamWidget).focus()

    def action_toggle_events(self) -> None:
        timeline = self.query_one(TimelineWidget)
        if timeline.has_class("collapsed"):
            timeline.remove_class("collapsed")
            timeline.focus()
        else:
            timeline.add_class("collapsed")
            self.query_one(StreamWidget).focus()

    def _refresh_widgets(self) -> None:
        self.query_one(TimelineWidget).set_state(self._state)
        self.query_one(StreamWidget).set_state(self._state)
        self.query_one(DetailWidget).set_state(self._state)

    def on_unmount(self) -> None:
        if self._session_recorder is not None:
            self._session_recorder.finalize()


def run_static_tui(events: list[TUIEvent]) -> None:
    AceAITUI(events).run()
